@@ -80,6 +80,7 @@ typedef struct _ExpoScreen
     DrawWindowProc             drawWindow;
     AddWindowGeometryProc      addWindowGeometry;
     DamageWindowRectProc       damageWindowRect;
+    DrawWindowTextureProc      drawWindowTexture;
 
     /*  Used for expo zoom animation */
     float expoCam;
@@ -123,6 +124,10 @@ typedef struct _ExpoScreen
     float curveDistance;
     float curveRadius;
 
+    GLfloat      *vpNormals;
+    GLfloat      *winNormals;
+    unsigned int winNormSize;
+
 } ExpoScreen;
 
 typedef struct _xyz_tuple
@@ -636,67 +641,39 @@ invertTransformedVertex (CompScreen              *s,
 
     alpha = -p1[2] / v[2];
 
-
     if (expoGetDeform (s->display) == DeformCurve && s->desktopWindowCount)
     {
-	/* If would be nice if someone would find a faster solution */
-	const float gapX  = expoGetVpDistance (s->display) * 0.1f * s->height /
-		            s->width * es->expoCam;
-	const float angle = es->curveAngle * DEG2RAD * (1.0 / (1.0 + gapX));
-	const float p     = sigmoidProgress (es->expoCam);
+	const float sws = s->width * s->width;
+	const float rs = (es->curveDistance * es->curveDistance) + 0.25;
+	const float p = ((2.0 * sws * (p1[2] - es->curveDistance) * v[2]) +
+			(2.0 * p1[0] * v[0]) - (v[0] * (float)s->width)) /
+			((v[2] * v[2] * sws) + (v[0] * v[0]));
+	const float q = (-(sws * rs) + (sws * (p1[2] - es->curveDistance) *
+			(p1[2] - es->curveDistance)) +
+                	(0.25 * sws) + (p1[0] * p1[0]) - (p1[0] * (float)s->width))
+			/ ((v[2] * v[2] * sws) + (v[0] * v[0]));
 
-	int   i = 10;
-	float a1, a2, *aM, x1, x2, *xM, d1, d2, *dM;
+	const float rq = (0.25 * p * p) - q;
+	const float ph = -p * 0.5;
 
-	a1 = -p1[2] / v[2];
-	a2 = (es->curveDistance - es->curveRadius - p1[2]) / v[2];
-
-	x1 = p1[0] + (a1 * v[0]);
-	x2 = p1[0] + (a2 * v[0]);
-
-	d1 = (es->curveDistance - (cos (angle / 2.0 - (x1 /
-		(float)s->width * angle)) * es->curveRadius)) * p;
-	d2 = (es->curveDistance - (cos (angle / 2.0 - (x2 /
-		(float)s->width * angle)) * es->curveRadius)) * p;
-
-	d1 = (d1 - p1[2]) / v[2];
-	d2 = (d2 - p1[2]) / v[2];
-
-	d1 = fabs (a1 - d1);
-	d2 = fabs (a2 - d2);
-
-	while (fabs(x1 - x2) > 1.0 && i--)
+	if (rq < 0.0)
 	{
-	    if (d1 < d2)
-	    {
-		aM = &a2;
-		dM = &d2;
-		xM = &x2;
-	    }
-	    else
-	    {
-		aM = &a1;
-		dM = &d1;
-		xM = &x1;
-	    }
-
-	    *aM = (a1 + a2) / 2.0;
-	    *xM = p1[0] + ((*aM) * v[0]);
-
-	    *dM = (es->curveDistance - (cos (angle / 2.0 - (*xM /
-		   (float)s->width * angle)) * es->curveRadius)) * p;
-	    *dM = (*dM - p1[2]) / v[2];
-
-	    *dM = fabs (*aM - *dM);
+	    vertex[0] = -1000;
+	    vertex[1] = -1000;
+	    return;
 	}
-
-	if (d1 < d2)
-	    alpha = a1;
 	else
-	    alpha = a2;
+	{
+	    alpha = ph + sqrt(rq);
+	    if (p1[2] + (alpha * v[2]) > 0.0)
+	    {
+		vertex[0] = -1000;
+		vertex[1] = -1000;
+		return;
+	    }
+	}
     }
 
-    
     vertex[0] = ceil (p1[0] + (alpha * v[0]));
     vertex[1] = ceil (p1[1] + (alpha * v[1]));
 }
@@ -801,6 +778,15 @@ expoPreparePaintScreen (CompScreen *s,
 		else
 		    es->vpActivity[vp] = MAX (0.0, es->vpActivity[vp] - val);
 	    }
+
+	for (i = 0; i < 360; i++)
+	{
+	    es->vpNormals[i * 3] = (-sin ((float)i * DEG2RAD) / s->width)
+				   * es->expoCam;
+	    es->vpNormals[(i * 3) + 1] = 0.0;
+	    es->vpNormals[(i * 3) + 2] = (-cos ((float)i * DEG2RAD) *
+					 es->expoCam) - (1 - es->expoCam);
+	}
     }
     else if (es->vpActivitySize)
     {
@@ -930,8 +916,12 @@ expoPaintWall (CompScreen              *s,
     matrixScale (&sTransform, oScale, oScale, 1.0);
 
     /* zoom out */
+    oScale = DEFAULT_Z_CAMERA / (camZ + DEFAULT_Z_CAMERA);
+    matrixScale (&sTransform, oScale, oScale, oScale);
+    glNormal3f (0.0, 0.0, -oScale);
     matrixTranslate (&sTransform, -camX, -camY, -camZ - DEFAULT_Z_CAMERA);
-    
+
+
     if (reflection)
     {
 	float scaleFactor = expoGetScaleFactor (s->display);
@@ -1074,6 +1064,8 @@ expoPaintWall (CompScreen              *s,
 	matrixTranslate (&sTransform, 0, - ((1.0 * sy) + gapY), 0.0f);
     }
 
+    glNormal3f (0.0, 0.0, -1.0);
+
     if (reflection)
     {
 	glEnable (GL_BLEND);
@@ -1273,36 +1265,53 @@ expoAddWindowGeometry (CompWindow *w,
 	REGION      reg;
 	GLfloat     *v;
 	int         offX = 0, offY = 0;
-	const float gapX = expoGetVpDistance (s->display) * 0.1f * s->height /
-		           s->width * es->expoCam;
-	const float angle = es->curveAngle * DEG2RAD * (1.0 / (1.0 + gapX));
-			
+	float       lastX, lastZ = 0.0;
+
+	const float radSquare = (es->curveDistance * es->curveDistance) + 0.25;
+	float       ang;
+	
 	reg.numRects = 1;
 	reg.rects = &reg.extents;
 
 	reg.extents.y1 = region->extents.y1;
 	reg.extents.y2 = region->extents.y2;
-	
-	x1 = (region->extents.x1 / EXPO_GRID_SIZE) * EXPO_GRID_SIZE;
+
 	x1 = region->extents.x1;
 	x2 = MIN (x1 + EXPO_GRID_SIZE, region->extents.x2);
 	
 	UNWRAP (es, s, addWindowGeometry);
-	while (x1 < region->extents.x2)
+	if (region->numRects > 1)
 	{
-	    reg.extents.x1 = x1;
-	    reg.extents.x2 = x2;
-
-	    XIntersectRegion (region, &reg, es->tmpRegion);
-
-	    if (!XEmptyRegion (es->tmpRegion))
+	    while (x1 < region->extents.x2)
 	    {
-		(*w->screen->addWindowGeometry) (w, matrix, nMatrix,
-						 es->tmpRegion, clip);
-	    }
+		reg.extents.x1 = x1;
+		reg.extents.x2 = x2;
 
-	    x1 = x2;
-	    x2 = MIN (x2 + EXPO_GRID_SIZE, region->extents.x2);
+		XIntersectRegion (region, &reg, es->tmpRegion);
+
+		if (!XEmptyRegion (es->tmpRegion))
+		{
+		    (*w->screen->addWindowGeometry) (w, matrix, nMatrix,
+						     es->tmpRegion, clip);
+		}
+
+		x1 = x2;
+		x2 = MIN (x2 + EXPO_GRID_SIZE, region->extents.x2);
+	    }
+	}
+	else
+	{
+	    while (x1 < region->extents.x2)
+	    {
+		reg.extents.x1 = x1;
+		reg.extents.x2 = x2;
+
+		(*w->screen->addWindowGeometry) (w, matrix, nMatrix,
+						 &reg, clip);
+
+		x1 = x2;
+		x2 = MIN (x2 + EXPO_GRID_SIZE, region->extents.x2);
+	    }
 	}
 	WRAP (es, s, addWindowGeometry, expoAddWindowGeometry);
 	
@@ -1315,16 +1324,29 @@ expoAddWindowGeometry (CompWindow *w,
                                         s->windowOffsetY, &offX, &offY);
 	}
 
+	lastX = -1000000000.0;
+	
 	for (i = oldVCount; i < w->vCount; i++)
 	{
-	    if (v[0] + offX >= -EXPO_GRID_SIZE && 
+	    if (v[0] == lastX)
+	    {
+		v[2] = lastZ;
+	    }
+	    else if (v[0] + offX >= -EXPO_GRID_SIZE && 
 		v[0] + offX < s->width + EXPO_GRID_SIZE)
 	    {
-		v[2] = es->curveDistance - (cos (angle / 2.0 - ((v[0] + offX) /
-		       (float)s->width * angle)) * es->curveRadius);
-		v[2] *= sigmoidProgress (es->expoCam);
+		ang = (((v[0] + offX) / (float)s->width) - 0.5);
+		ang *= ang;
+		if (ang < radSquare)
+		{
+		    v[2] = es->curveDistance - sqrt (radSquare - ang);
+		    v[2] *= sigmoidProgress (es->expoCam);
+		}
 	    }
 
+	    lastX = v[0];
+	    lastZ = v[2];
+
 	    v += w->vertexStride;
 	}
     }
@@ -1336,6 +1358,81 @@ expoAddWindowGeometry (CompWindow *w,
     }
 }
 
+static void
+expoDrawWindowTexture (CompWindow	    *w,
+		       CompTexture	    *texture,
+		       const FragmentAttrib *attrib,
+		       unsigned int	    mask)
+{
+    CompScreen *s = w->screen;
+
+    EXPO_SCREEN (s);
+
+    if (es->expoCam > 0.0 && expoGetDeform (s->display) == DeformCurve &&
+        s->desktopWindowCount && s->lighting)
+    {
+	int     i, idx;
+	int     offX = 0, offY = 0;
+	float   x;
+	GLfloat *v;
+
+	if (es->winNormSize < w->vCount * 3)
+	{
+	    es->winNormals = realloc (es->winNormals,
+				       w->vCount * 3 * sizeof (GLfloat));
+	    if (!es->winNormals)
+	    {
+		es->winNormSize = 0;
+		return;
+	    }
+	    es->winNormSize = w->vCount * 3;
+	}
+	
+	if (!windowOnAllViewports (w))
+	{
+	    getWindowMovementForOffset (w, s->windowOffsetX,
+                                        s->windowOffsetY, &offX, &offY);
+	}
+	
+	v = w->vertices + (w->vertexStride - 3);
+	
+	for (i = 0; i < w->vCount; i++)
+	{
+	    x = ((float)(v[0] + offX - (s->width / 2)) * es->curveAngle) /
+		(float)s->width;
+	    while (x < 0)
+		x += 360.0;
+
+	    idx = floor (x);
+
+	    es->winNormals[i * 3] = -es->vpNormals[idx * 3];
+	    es->winNormals[(i * 3) + 1] = es->vpNormals[(idx * 3) + 1];
+	    es->winNormals[(i * 3) + 2] = es->vpNormals[(idx * 3) + 2];
+
+	    v += w->vertexStride;
+	}
+	
+	glEnable (GL_NORMALIZE);
+	glNormalPointer (GL_FLOAT,0, es->winNormals);
+	
+	glEnableClientState (GL_NORMAL_ARRAY);
+	
+	UNWRAP (es, s, drawWindowTexture);
+	(*s->drawWindowTexture) (w, texture, attrib, mask);
+	WRAP (es, s, drawWindowTexture, expoDrawWindowTexture);
+
+	glDisable (GL_NORMALIZE);
+	glDisableClientState (GL_NORMAL_ARRAY);
+	glNormal3f (0.0, 0.0, -1.0);
+	return;
+    }
+glEnable (GL_NORMALIZE);
+    UNWRAP (es, s, drawWindowTexture);
+    (*s->drawWindowTexture) (w, texture, attrib, mask);
+    WRAP (es, s, drawWindowTexture, expoDrawWindowTexture);
+    glDisable (GL_NORMALIZE);
+}
+
 static Bool
 expoPaintWindow (CompWindow              *w,
 		 const WindowPaintAttrib *attrib,
@@ -1601,8 +1698,11 @@ expoInitScreen (CompPlugin *p,
 
     es->tmpRegion = XCreateRegion ();
     if (!es->tmpRegion)
+    {
+	free (es);
 	return FALSE;
-
+    }
+    
     es->anyClick  = FALSE;
     es->vpUpdateMode = VPUpdateNone;
 
@@ -1611,8 +1711,9 @@ expoInitScreen (CompPlugin *p,
 
     es->grabIndex = 0;
 
-    es->expoCam  = 0.0f;
-    es->expoMode = 0;
+    es->expoActive = FALSE;
+    es->expoCam    = 0.0f;
+    es->expoMode   = 0;
 
     es->dndState  = DnDNone;
     es->dndWindow = NULL;
@@ -1623,6 +1724,13 @@ expoInitScreen (CompPlugin *p,
     es->vpActivity     = NULL;
     es->vpActivitySize = 0;
 
+    es->vpNormals = malloc (360 * 3 * sizeof (GLfloat));
+    if (!es->vpNormals)
+	return FALSE;
+
+    es->winNormals  = NULL;
+    es->winNormSize = 0;
+
     WRAP (es, s, paintOutput, expoPaintOutput);
     WRAP (es, s, paintScreen, expoPaintScreen);
     WRAP (es, s, donePaintScreen, expoDonePaintScreen);
@@ -1632,6 +1740,7 @@ expoInitScreen (CompPlugin *p,
     WRAP (es, s, damageWindowRect, expoDamageWindowRect);
     WRAP (es, s, paintWindow, expoPaintWindow);
     WRAP (es, s, addWindowGeometry, expoAddWindowGeometry);
+    WRAP (es, s, drawWindowTexture, expoDrawWindowTexture);
 
     s->base.privates[ed->screenPrivateIndex].ptr = es;
 
@@ -1661,6 +1770,7 @@ expoFiniScreen (CompPlugin *p,
     UNWRAP (es, s, damageWindowRect);
     UNWRAP (es, s, paintWindow);
     UNWRAP (es, s, addWindowGeometry);
+    UNWRAP (es, s, drawWindowTexture);
 
     free (es);
 }
