@@ -37,7 +37,7 @@
 #include <cairo.h>
 
 #define PI 3.14159265359f
-#define VIEWPORT_SWITCHER_SIZE 70
+#define VIEWPORT_SWITCHER_SIZE   100
 #define ARROW_SIZE 33
 
 #define WIN_X(w) ((w)->attrib.x - (w)->input.left)
@@ -108,6 +108,7 @@ typedef struct _WallScreen
     int boxTimeout;
     int boxOutputDevice;
 
+    int grabIndex;
     int timer;
 
     Window moveWindow;
@@ -116,11 +117,11 @@ typedef struct _WallScreen
     WindowPaintAttrib mSAttribs;
     float             mSzCamera;
 
-    float firstViewportX;
-    float firstViewportY;
-    float viewportWidth;
-    float viewportHeight;
-    float viewportBorder;
+    int firstViewportX;
+    int firstViewportY;
+    int viewportWidth;
+    int viewportHeight;
+    int viewportBorder;
 
     int moveWindowX;
     int moveWindowY;
@@ -166,8 +167,6 @@ wallClearCairoLayer (cairo_t *cr)
     cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
     cairo_paint (cr);
     cairo_restore (cr);
-    cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-    cairo_save (cr);
 }
 
 static void
@@ -175,10 +174,9 @@ wallDrawSwitcherBackground (CompScreen *s)
 {
     cairo_t         *cr;
     cairo_pattern_t *pattern;
-    float           border = 10.0f, outline = 2.0f;
-    float           width, height, radius;
+    float           outline = 2.0f;
+    int             width, height, radius;
     float           r, g, b, a;
-    int             viewportWidth, viewportHeight;
     int             i, j;
 
     WALL_SCREEN (s);
@@ -186,9 +184,10 @@ wallDrawSwitcherBackground (CompScreen *s)
     cr = ws->switcherContext.cr;
     wallClearCairoLayer (cr);
 
-    width = (float) ws->switcherContext.width - outline;
-    height = (float) ws->switcherContext.height - outline;
+    width = ws->switcherContext.width - outline;
+    height = ws->switcherContext.height - outline;
 
+    cairo_save (cr);
     cairo_translate (cr, outline / 2.0f, outline / 2.0f);
 
     /* set the pattern for the switcher's background */
@@ -202,7 +201,7 @@ wallDrawSwitcherBackground (CompScreen *s)
     cairo_set_source (cr, pattern);
 
     /* draw the border's shape */
-    radius = (float) wallGetEdgeRadius (s->display);
+    radius = wallGetEdgeRadius (s->display);
     if (radius)
     {
 	cairo_arc (cr, radius, radius, radius, PI, 1.5f * PI);
@@ -223,41 +222,35 @@ wallDrawSwitcherBackground (CompScreen *s)
     cairo_set_line_width (cr, outline);
     getColorRGBA (OutlineColor, s->display);
     cairo_set_source_rgba (cr, r, g, b, a);
-    cairo_stroke (cr);
+    cairo_stroke(cr);
 
     cairo_pattern_destroy (pattern);
+    cairo_restore (cr);
 
-    viewportWidth  = floor ((width  - border * (s->hsize + 1)) / s->hsize -
-			    outline / 2.0f);
-    viewportHeight = floor ((height - border * (s->vsize + 1)) / s->vsize -
-			    outline / 2.0f);
-
-    ws->viewportWidth = viewportWidth;
-    ws->viewportHeight = viewportHeight;
-    ws->viewportBorder = border;
-
-    cairo_translate (cr, border, border);
-
-    for (i = 0; i < s->hsize; i++)
+    cairo_save (cr);
+    for (i = 0; i < s->vsize; i++)
     {
-	for (j = 0; j < s->vsize; j++)
+	cairo_translate (cr, 0.0, ws->viewportBorder);
+	cairo_save (cr);
+	for (j = 0; j < s->hsize; j++)
 	{
-	    float vpX, vpY;
-
-	    vpX = i * (viewportWidth + border + outline / 2.0f);
-	    vpY = j * (viewportHeight + border + outline / 2.0f);
+	    cairo_translate (cr, ws->viewportBorder, 0.0);
 
 	    /* this cuts a hole into our background */
 	    cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
 	    cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 1.0);
-	    cairo_rectangle (cr, vpX, vpY, viewportWidth, viewportHeight);
+	    cairo_rectangle (cr, 0, 0, ws->viewportWidth, ws->viewportHeight);
 
 	    cairo_fill_preserve (cr);
 	    cairo_set_operator (cr, CAIRO_OPERATOR_XOR);
 	    cairo_fill (cr);
-	}
-    }
 
+	    cairo_translate (cr, ws->viewportWidth, 0.0);
+	}
+	cairo_restore(cr);
+
+	cairo_translate (cr, 0.0, ws->viewportHeight);
+    }
     cairo_restore (cr);
 }
 
@@ -267,23 +260,16 @@ wallDrawThumb (CompScreen *s)
     cairo_t         *cr;
     cairo_pattern_t *pattern;
     float           r, g, b, a;
-    float           border = 10.0f, outline = 2.0f;
-    float           width, height;
+    float           outline = 2.0f;
+    int             width, height;
 
     WALL_SCREEN(s);
 
     cr = ws->thumbContext.cr;
     wallClearCairoLayer (cr);
 
-    width  = (float) ws->thumbContext.width;
-    height = (float) ws->thumbContext.height;
-
-    ws->viewportWidth = width;
-    ws->viewportHeight = height;
-    ws->viewportBorder = border;
-
-    width -= outline;
-    height -= outline;
+    width  = ws->thumbContext.width - outline;
+    height = ws->thumbContext.height - outline;
 
     cairo_translate (cr, outline / 2.0f, outline / 2.0f);
 
@@ -313,7 +299,7 @@ wallDrawHighlight(CompScreen *s)
 {
     cairo_t         *cr;
     cairo_pattern_t *pattern;
-    float           width, height;
+    int             width, height;
     float           r, g, b, a;
     float           outline = 2.0f;
 
@@ -323,8 +309,8 @@ wallDrawHighlight(CompScreen *s)
     cr = ws->highlightContext.cr;
     wallClearCairoLayer (cr);
 
-    width  = (float) ws->highlightContext.width - outline;
-    height = (float) ws->highlightContext.height - outline;
+    width  = ws->highlightContext.width - outline;
+    height = ws->highlightContext.height - outline;
 
     cairo_translate (cr, outline / 2.0f, outline / 2.0f);
 
@@ -488,11 +474,15 @@ wallComputeTranslation (CompScreen *s,
 			float      *x,
 			float      *y)
 {
-    float dx, dy, elapsed;
+    float dx, dy, elapsed, duration;
 
     WALL_SCREEN (s);
 
-    elapsed = 1 - (ws->timer / (wallGetSlideDuration (s->display) * 1000));
+    duration = wallGetSlideDuration (s->display) * 1000.0;
+    if (duration != 0.0)
+	elapsed = 1.0 - (ws->timer / duration);
+    else
+	elapsed = 1.0;
 
     if (elapsed < 0.0)
 	elapsed = 0.0;
@@ -550,6 +540,9 @@ wallMoveViewport (CompScreen *s,
 	ws->gotoX = s->x - x;
 	ws->gotoY = s->y - y;
 
+	if (!ws->grabIndex)
+	    ws->grabIndex = pushScreenGrab (s, s->invisibleCursor, "wall");
+
 	moveScreenViewport (s, x, y, TRUE);
 
 	ws->moving = TRUE;
@@ -563,7 +556,8 @@ wallMoveViewport (CompScreen *s,
 	else
 	    ws->boxTimeout = 0;
 
-	if (otherScreenGrabExist (s, "move", "scale", "group-drag", "wall", 0))
+	if (otherScreenGrabExist (s, "move", "scale", "switcher",
+				  "group-drag", "wall", 0))
 	{
 	    ws->boxTimeout = 0;
 	    ws->moving = FALSE;
@@ -1022,7 +1016,7 @@ wallDrawCairoTextureOnScreen (CompScreen *s)
     float      centerX, centerY;
     float      width, height;
     float      topLeftX, topLeftY;
-    float      border = 10.0f;
+    float      border;
     int        i, j;
     CompMatrix matrix;
     BOX        box;
@@ -1037,6 +1031,7 @@ wallDrawCairoTextureOnScreen (CompScreen *s)
     centerY = s->outputDev[ws->boxOutputDevice].region.extents.y1 +
 	      (s->outputDev[ws->boxOutputDevice].height / 2.0f);
 
+    border = (float) ws->viewportBorder;
     width  = (float) ws->switcherContext.width;
     height = (float) ws->switcherContext.height;
 
@@ -1408,6 +1403,12 @@ wallPreparePaintScreen (CompScreen *s,
 	    wallReleaseMoveWindow (s);
 	else
 	    focusDefaultWindow (s);
+
+	if (ws->grabIndex)
+	{
+	    removeScreenGrab (s, ws->grabIndex, NULL);
+	    ws->grabIndex = 0;
+	}
     }
 
     UNWRAP (ws, s, preparePaintScreen);
@@ -1424,11 +1425,14 @@ wallPaintTransformedOutput (CompScreen              *s,
 			    unsigned int            mask)
 {
     WALL_SCREEN (s);
+    Bool clear = (mask & PAINT_SCREEN_CLEAR_MASK);
 
     if (ws->miniScreen)
     {
 	CompTransform sTransform = *transform;
 
+	mask &= ~PAINT_SCREEN_CLEAR_MASK;
+
 	/* move each screen to the correct output position */
 	matrixTranslate (&sTransform,
 			 -output->region.extents.x1 / output->width,
@@ -1474,7 +1478,8 @@ wallPaintTransformedOutput (CompScreen              *s,
 	float         px, py;
 	int           tx, ty;
 
-	clearTargetOutput (s->display, GL_COLOR_BUFFER_BIT);
+	if (clear)
+	    clearTargetOutput (s->display, GL_COLOR_BUFFER_BIT);
 
 	px = ws->curPosX;
 	py = ws->curPosY;
@@ -1615,14 +1620,15 @@ static void
 wallCreateCairoContexts (CompScreen *s,
 			 Bool       initial)
 {
-    float border = 10.0f;
-    float width, height;
+    int width, height;
 
     WALL_SCREEN (s);
 
-    width = VIEWPORT_SWITCHER_SIZE * s->hsize + (2 * border * (s->hsize - 1));
-    width *= (float)s->width / (float)s->height;
-    height = VIEWPORT_SWITCHER_SIZE * s->vsize + (2 * border * (s->vsize - 1));
+    ws->viewportWidth = VIEWPORT_SWITCHER_SIZE * (float)wallGetPreviewScale(s->display) / 100.0f;
+    ws->viewportHeight = ws->viewportWidth * (float)s->height / (float)s->width;
+    ws->viewportBorder = wallGetBorderWidth(s->display);
+    width = s->hsize * (ws->viewportWidth + ws->viewportBorder) + ws->viewportBorder;
+    height = s->vsize * (ws->viewportHeight + ws->viewportBorder) + ws->viewportBorder;
 
     wallDestroyCairoContext (s, &ws->switcherContext);
     ws->switcherContext.width = width;
@@ -1631,14 +1637,14 @@ wallCreateCairoContexts (CompScreen *s,
     wallDrawSwitcherBackground (s);
 
     wallDestroyCairoContext (s, &ws->thumbContext);
-    ws->thumbContext.width = (width - border * (s->hsize + 1)) / s->hsize;
-    ws->thumbContext.height = (height - border * (s->vsize + 1)) / s->vsize;
+    ws->thumbContext.width = ws->viewportWidth;
+    ws->thumbContext.height = ws->viewportHeight;
     wallSetupCairoContext (s, &ws->thumbContext);
     wallDrawThumb (s);
 
     wallDestroyCairoContext (s, &ws->highlightContext);
-    ws->highlightContext.width = (width - border * (s->hsize + 1)) / s->hsize;
-    ws->highlightContext.height = (height - border * (s->vsize + 1)) / s->vsize;
+    ws->highlightContext.width = ws->viewportWidth;
+    ws->highlightContext.height = ws->viewportHeight;
     wallSetupCairoContext (s, &ws->highlightContext);
     wallDrawHighlight (s);
 
@@ -1677,6 +1683,12 @@ wallDisplayOptionChanged (CompDisplay        *display,
 	    wallDrawSwitcherBackground (s);
 	break;
 
+    case WallDisplayOptionBorderWidth:
+    case WallDisplayOptionPreviewScale:
+	for (s = display->screens; s; s = s->next)
+	    wallCreateCairoContexts (s, FALSE);
+	break;
+
     case WallDisplayOptionThumbGradientBaseColor:
     case WallDisplayOptionThumbGradientHighlightColor:
 	for (s = display->screens; s; s = s->next)
@@ -1822,6 +1834,8 @@ wallInitDisplay (CompPlugin  *p,
     wallSetFlipDownEdgeInitiate (d, wallFlipDown);
 
     wallSetEdgeRadiusNotify (d, wallDisplayOptionChanged);
+    wallSetBorderWidthNotify (d, wallDisplayOptionChanged);
+    wallSetPreviewScaleNotify (d, wallDisplayOptionChanged);
     wallSetOutlineColorNotify (d, wallDisplayOptionChanged);
     wallSetBackgroundGradientBaseColorNotify (d, wallDisplayOptionChanged);
     wallSetBackgroundGradientHighlightColorNotify (d, wallDisplayOptionChanged);
@@ -1865,6 +1879,7 @@ wallInitScreen (CompPlugin *p,
 	return FALSE;
 
     ws->boxTimeout = 0;
+    ws->grabIndex = 0;
     ws->moving = FALSE;
     ws->showPreview = FALSE;
 
